/*********************************************************************
 *     Humidity Sensor Module Firmware
 *********************************************************************
 * FileName:        main.c
 * Dependencies:    none
 * Processor:       Microchip PIC18(L)F14K22
 * Compiler:        Microchip XC8 V1.20
 *
 * SOFTWARE IS PROVIDED "AS IS."  DIGI-KEY EXPRESSLY DISCLAIMS ANY
 * WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL DIGI-KEY
 * BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
 * ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
 * DIGI-KEY ALSO DISCLAIMS ANY LIABILITY FOR PATENT OR COPYRIGHT
 * INFRINGEMENT.
 *
 * Author: K. Culkins
 * Version: Initial Release v1.00
 * Date: 5 Nov, 2013
 *               
 * Revisions:
 * By              Date            Comment
 * ------------------------------------------------------------------
 *
 *
 ********************************************************************/

/***********************************************************************
 *  main.c
 *
 *  Main function for humidity sampling software. Humidity sensor is a
 *  Honeywell HIH-5030.  Additional project functions perform, data
 *  processing and parsing, and serial communications.
 *
 ***********************************************************************/
#include <xc.h>
#include <plib.h>

#include "main.h"
#include "serial.h"
#include "monitor.h"
#include "sensor.h"

#pragma config WDTEN = OFF
#pragma config FOSC = IRC
#pragma config MCLRE = ON
#pragma config LVP = OFF

// global container for the sensor data
// structure is defined in sensor.h
struct sensor sensor;

void main(void)
{
#ifdef ONEMHZ         // clock speed definitions in main.h
  // Set internal oscillator to 1MHz
  OSCCON = 0x32;
#elif defined(FOURMHZ)
  // Set internal oscillator to 4Mhz
  OSCCON = 0x52;
#elif defined(EIGHTMHZ)
  // Set internal oscillator to 8Mhz
  OSCCON = 0x62;
#endif

  // Initialize ports
  Init_ports();

  // Start serial communications, RS232.
  // OpenUSART reconfigures RC6 and RC7, reconfigured in Usart_init (serial.c).
  Usart_init();

  // Init ADC
  ADC_init();
  
  // Enter monitor 
  Monitor();

  Reset();

}

/***********************************************************************
 *  Init_ports()
 *
 *  Prototype: (void) Init_ports(void)
 *  Passed arguments: none
 *  Return value: none
 *
 *  Description:  Configure and initialize PIC18 ports
 *
 *  RC3, Input, Analog, samples humidity sensor output. (sensor pin 2)
 *  RC6, Input, Analog, sample Vdd powering the humidity sensor (Vdd from the PIC, PIC pin 8)
 *       Configured in Serial.c
 *  RC7, Output, Power the humidity sensor (PIC pin 9)
 *       Configured and initialized in Serial.c
 * 
 *  RB5, Input, RX from host serial port
 *       Initialization performed by the UART init routine (PIC pin 12)
 *  RB7, Output, TX to host serial port
 *       Initialization performed by the UART init routine (PIC pin 10)
 *
 ***********************************************************************/
void Init_ports(void) {

  // Configure port pins RC3 and RC6.  Remainder digital I/O,
  ANSEL = 0x80;  // RC3 analog
  ANSELH = 0x01;  // RC6 analog

  // Initialize the ports, port pin direction and state.
  LATCbits.LATC2 = 0;

  // port initial state. Humidity sensor; power is removed.
  LATCbits.LATC7 = 0;

  TRISCbits.TRISC3 = INPUT;
  TRISCbits.TRISC6 = INPUT;
  TRISCbits.TRISC7 = OUTPUT;
   
  TRISBbits.TRISB5 = INPUT;
  TRISBbits.TRISB7 = INPUT;


  return;
}

/***********************************************************************
 *  ADC_init()
 *
 *  Prototype: (void) ADC_init(void)
 *  Passed arguments: none
 *  Return value: none
 *
 *  Description:  Configure PIC18 Analog-to-Digital Converter
 ***********************************************************************/
void ADC_init(void) {

  // Default channel AN0 selected
  // ADC is disabled
  ADCON0 = 0x00;
  // Vref+ and Vref- internal
  ADCON1 = 0x00;

  #ifdef ONEMHZ
  // Conversion Clock Fosc/2
  // Acquisition 2 TAD
  // Right Justified
  ADCON2 = 0x88;
  #elif defined(FOURMHZ)
  // Conversion Clock Fosc/8
  ADCON2 = 0x89;
  #elif defined(EIGHTMHZ)
  // Conversion Clock Fosc/16
  ADCON2 = 0x8D;
  #endif

  return;
}