/*********************************************************************
 *     Humidity Sensor Module Firmware
 *********************************************************************
 * FileName:        monitor.c
 * Dependencies:    none
 * Processor:       Microchip PIC18(L)F14K22
 * Compiler:        Microchip XC8 V1.20
 *
 * SOFTWARE IS PROVIDED "AS IS."  DIGI-KEY EXPRESSLY DISCLAIMS ANY
 * WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL DIGI-KEY
 * BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
 * ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
 * DIGI-KEY ALSO DISCLAIMS ANY LIABILITY FOR PATENT OR COPYRIGHT
 * INFRINGEMENT.
 *
 * Author: K. Culkins
 * Version: Initial Release v1.00
 * Date: 15 Oct, 2013
 *                
 * Revisions:
 * By              Date            Comment
 * ------------------------------------------------------------------
 *
 *
 ********************************************************************/

/***********************************************************************
 *
 *  monitor.c
 *  Monitor program responds to commands from XBEE RF module.
 *
 ***********************************************************************/

#include <xc.h>
#include <plib.h>
#include <stdio.h>
#include <string.h>

#include "monitor.h"
#include "sensor.h"

extern struct sensor sensor;

char Connect[6] = "*";
char Error1[8] = "Error";

/***********************************************************************
 *  Monitor()
 *
 *  Prototype: (void) Monitor(void)
 *  Passed arguments: none
 *  Return value: none
 *
 *  Description:  Monitor function responds to command requests for 
 *  pressure, temperature, and humidity data and controls the sleep mode.
 ***********************************************************************/
void Monitor(void) {

  char temp;

  // Temperature in Celsius
  // Pressure in hecta Pascals.

  while (1) {

    Delay10KTCYx((unsigned char)1);

    // USART Sleep 
    // Prior to entering sleep, check the RCIDL (receive idle) bit,
    // If no receive is in progress, set the WUE (wake up enable) bit
    // The WUE is cleared by hardware after exiting sleep
    while(!BAUDCONbits.RCIDL);  // wait for receiver idle before going to sleep

    // Set the Wake Up Event bit and enable the usart interrupt,
    // then enter sleep. Read the USART Receive FIFO to clear any interrupt flag

    BAUDCONbits.WUE = 1;
    PIE1bits.RCIE = 1;
    
    Sleep();
    Nop();
    Nop();

    // Disable Receive interrupt
    PIE1bits.RCIE = 0;

    // For the USART wakeup from sleep to function properly, the first byte received 
    // to wake up the device is a break character which should be followed by the
    // command character.  The break character is discarded and the command character
    // is evaluated.
    while (!DataRdyUSART());  // return value '1' if data is available
    // Discard first byte after coming out of sleep
    temp = getcUSART();

    // wait for next byte
    while (!DataRdyUSART());  // return value '1' if data is available

    // read receiver register
    temp = getcUSART();

    // evaluate the RX character
    if (temp == 'H' || temp == 'h') {
      // if 'H'ello, respond with connection ack character 
      putsUSART(Connect);
    } 
    else if (temp == 'R' || temp == 'r') {

      // Sample data from Honeywell sensor, calculate humidity, and format data
      // The temperature calculated in the function above, Baro_temp_process(), is
      // used in the humidity (temperature) compensation calculation in
      // the function Humidity_sensorsample() below.
      Humidity_sensorsample();
      // Send data (data is formatted in sensor.c function).
      putsUSART(sensor.humidity);
    }
    // if not (h)ello or (r)equest, respond with error characters
    else { 

      putsUSART(Error1);

    }  

  }  // end while(1)

  return;

}

