/*********************************************************************
 *     Humidity Sensor Module Firmware
 *********************************************************************
 * FileName:        sensor.c
 * Dependencies:    none
 * Processor:       Microchip PIC18(L)F14K22
 * Compiler:        Microchip XC8 V1.20
 *
 * SOFTWARE IS PROVIDED "AS IS."  DIGI-KEY EXPRESSLY DISCLAIMS ANY
 * WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL DIGI-KEY
 * BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
 * ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
 * DIGI-KEY ALSO DISCLAIMS ANY LIABILITY FOR PATENT OR COPYRIGHT
 * INFRINGEMENT.
 *
 * Author: K. Culkins
 * Version: Initial Release v1.00
 * Date: 15 Oct, 2013
 *                
 * Revisions:
 * By              Date            Comment
 * ------------------------------------------------------------------
 *
 *
 ********************************************************************/

/***********************************************************************
 * sensor.c
 * 
 * Functions to read sensor calibration values, sample  pressure/temperature,
 * and humidity sensors, and process sampled data.
 *
 * NOTE:
 * This source was originally developed and compiled using the C18 compiler.
 * Because C18 didn't have a format conversion operator for printing float
 * data types other means were required to convert and manipulate the float data
 * into printable characters.  This code is now compiled with the XC8 compiler.
 * The XC8 compiler does provide format conversion operators for float data,
 * but the program code grew unacceptably.  So, the float conversion
 * operator work-around for the C18 compiler has been retained with a significant
 * reduction in program memory usage.  In fact, a trial of the float operator in
 * an sprintf() statement nearly max'ed out the program memory space, from 53%
 * used, to 98% used.
 *
 * The method used here breaks apart the float into its integral and
 * fractional components, using the conversion operators other than float to format
 * the converted data. A small downside, this method will result in additional
 * rounding errors.
 * 
 * For example, if the temperature is 21.32C degrees, the fractional part of 
 * the temperature is 0.32 degree, but the modf function returns 0.319997 (the 
 * modulus or fractional). The fractional component is still a float data type
 * and requires further manipulation to use existing integer conversion operators.
 * Performing a decimal shift to the left, multiply by 100, and type cast as 
 * an unsigned int will now leave the data as 31, having lost the lower bits
 * and resulting in further degradation of precision.
 *  
 * It is important to thoroughly evaluate float data manipulation and results 
 * for every implementation to determine the loss in precision and it's impact
 * on the application.
 *
 ***********************************************************************/
#include <xc.h>
#include <plib.h>
#include <stdio.h>
#include <math.h>

#include "sensor.h"

// 1=ADC on, 0=ADC off
#define ADC_ON ADCON0bits.ADON = 1
#define ADC_OFF ADCON0bits.ADON = 0;
// 1=power applied, 0=power off
#define ENABLE_HUMID_POWER LATCbits.LATC7 = 1;
#define DISABLE_HUMID_POWER LATCbits.LATC7 = 0;

//globals
extern struct sensor sensor;

/***********************************************************************
 *  Humidity_sensorsample()
 *
 *  Prototype: (void) Humidity_sensorsample(void)
 *  Passed arguments: none
 *  Return value: none
 *
 *  Description:  Sample the analog signal from the Honeywell HIH-5031 humidity
 *                sensor and calculate the humidity from the sampled data
 ***********************************************************************/
void Humidity_sensorsample(void) {

  unsigned int ref_sample,
               humid_sample,
               humid_upper;
  float        H1,
               sensor_RH,
               true_RH,
               humid_integer,
               humid_mod;

  // Apply power to the humidity sensor
  ENABLE_HUMID_POWER;
  // delay 30ms for settling
  Delay1KTCYx((unsigned char)60);

  // Turn on ADC.
  ADC_ON;
  // Wait for ADC
  Delay10TCYx(5);
   
  // Sample humidity sensor applied voltage.
  // Select channel AN8. and Go
  SelChanConvADC(ADC_CH8);

  // wait for acquisition and conversion to complete
  while(BusyADC());

  // Read sample.
  ref_sample = ReadADC();

  // Sample humidity sensor.
  // Select channel AN7. and Go
  SelChanConvADC(ADC_CH7);

  // wait for acquisition and conversion to complete
  while(BusyADC());

  // Read sample.
  humid_sample = ReadADC();

  // Turn off the ADC
  ADC_OFF;

  // If there are problems with the sensor, print "nn" and return;
  if(humid_sample > ref_sample) {
    sprintf(sensor.humidity,"Hnn");
    return;
  }
  // Calculate the relative humidity
  H1 = 157.23*(float)humid_sample/(float)ref_sample;
  if(H1 <23.82) sensor_RH=0;
  else sensor_RH = H1-23.82;

  // This fixed temperature value 25 (degrees in Celsius) is used in the temperature
  // compensated relative humidity calculation below.  If a temperature sensor
  // were available, the sampled temperature would be assigned to this variable.
  sensor.comp_temperature = 25;

  // Calculate temperature compensated relative humidity.
  true_RH = sensor_RH/(1.0546-0.00216*sensor.comp_temperature);

  // Limit max to 99% humidity, to prevent overrun  
  // of string data
  if(true_RH > 99.0)true_RH = 99.0;

  // Separate the integral and fractional parts of the humidity sensor data.
  // This code was originally written with the C18 compiler and no float conversion
  // operator formatting was available.  The XC8 compiler offers float conversion,
  // but the program memory usage is large.  So, this manual method of converting
  // a float value to a string is used.
  humid_mod = modf((double) true_RH,(double *) &humid_integer);

  // Convert the integer part of the float
  humid_upper = (unsigned int)humid_integer;

  // format data
  sprintf(sensor.humidity,"RH %02.2u%%",humid_upper);

  // Remove power from Humidity sensor
  DISABLE_HUMID_POWER;

  return;
}

